package indep_set.root;

import java.util.ArrayList;
import java.util.List;

/**
 * A root can be represented by a string of the form
 * "332 13 23"
 * The first word contains the out-degrees of the level-1 vertices
 * Each subsequent word represents a level-2 vertex: each index in the word
 * represents the index of the level-1 vertex that the gien level-2 vertex is
 * adjacent to.
 */
public class RootCode {
	
	String code;
	List<Integer> degrees; //lvl-1 vertices out-degress
	List<List<Integer>> joins = new ArrayList<List<Integer>>();
	
	int lvl1size, lvl2size;
	Root root;
	
	public RootCode(String code){
		this.code = code;
	}
	
	public Root getRoot(){
		parse();
		calculateSize();
		root = new Root(lvl1size, lvl2size);
		
		int [] remaingDegrees = new int[lvl1size];
		for (int i = 0; i < lvl1size; i++){
			remaingDegrees[i] = degrees.get(i);
		}
		
		for (int i = 0; i < joins.size(); i++){
			List<Integer> join = joins.get(i);
			for (int a : join){
				remaingDegrees[a]--;
				root.addEdge(a, i);
			}
		}
		
		int j = joins.size();
		for (int a = 0; a < lvl1size; a++){
			while (remaingDegrees[a]-- > 0){
				root.addEdge(a, j++);
			}				
		}
		root.code = code;
				
		
		assert(j == lvl2size);
		
		return root;
	}
	
	//Converts string to lists of integers: degrees, joins
	private void parse(){
		String[] codeTokens = code.split(" ");
		degrees = parseDegree(codeTokens[0]);
		for (int i = 1; i < codeTokens.length; i++){
			joins.add(parseJoin(codeTokens[i]));
		}
	}
	
	private void calculateSize(){
		lvl1size = degrees.size();
		lvl2size = 0;
		for (int d : degrees){
			lvl2size += d; 
		}
		for (List<Integer> join : joins){
			lvl2size -= join.size() - 1;
		}
	}
	
	
	private List<Integer> parseDegree(String word){
		List<Integer> list = new ArrayList<Integer>();
		for (int i = 0; i < word.length(); i++){
			list.add(Integer.valueOf(word.substring(i, i+1)));
		}
		return list;
	}
	
	private List<Integer> parseJoin(String word){
		List<Integer> list = new ArrayList<Integer>();
		for (int i = 0; i < word.length(); i++){
			list.add(Integer.valueOf(word.substring(i, i+1)) - 1);
		}
		return list;
	}

}

