package indep_set.run;

import indep_set.Config;
import indep_set.LaTeX.LaTeXTemplate;
import indep_set.compound.DataPagePrinter;
import indep_set.root.RootCode;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Generates data tables for all level 0,1,2,3 configurations
 * Publishes to 
 * latex\data.tex
 */
public class RunDataPages {
		
	File inFile;
	File outFile;
	File templateFile = new File("latex\\template.tex");
		
	public RunDataPages(File inFile, File outFile){
		this.inFile = inFile;
		this.outFile = outFile;
	}
	
	public void run() throws IOException{
		BufferedReader reader = null;
		PrintWriter writer = null;
		BufferedReader templateReader = null;
		LaTeXTemplate template = null;
		
		try {
			reader = new BufferedReader(new FileReader(inFile));
			writer = new PrintWriter(new BufferedWriter(new FileWriter(outFile)));
			templateReader = new BufferedReader(new FileReader(templateFile));
			template = new LaTeXTemplate(templateReader);
			
			writer.println(template.header());
			writer.println(Config.configLaTeX());
			
            String l;
            while ((l = reader.readLine()) != null) {
            	if (l.trim().isEmpty()) continue;
            	if (l.startsWith("%")) continue;
            	new DataPagePrinter(new RootCode(l).getRoot(), writer).print();
            }
            
            writer.println(template.footer());
        } finally {
            if (reader != null) {
            	reader.close();
            }
            if (writer !=null){
            	writer.close();
            }
            if (templateReader!=null){
            	templateReader.close();
            }
        }
	}
	
	
	public static void runRegular(){
		File inFile = new File(RunGenerateRootCodes.outFileName());
		File outFile = new File("latex\\data.tex");
		try{
			new RunDataPages(inFile, outFile).run();
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	public static void runSpecial(){
		File inFile = new File("rootCodesSpecial.txt");
		File outFile = new File("latex\\dataSpecial.tex");
		try{
			new RunDataPages(inFile, outFile).run();
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) throws IOException{
		runRegular();
	}
}
